/*! \file can_api.h

  \par Datei: 
  CAN_API.CPP \n
  oder \n
  CAN_API.DLL
  
  \par Header: 
  CAN_API.H \n
  CANPAR.H


  \brief Auflistung der Funktionen

Die Beschreibung soll helfen, die Treiberfunktionen direkt - also ohne das canParActiveX/CaniX - zu verwenden. 
Dies wird hiermit auch ausdrcklich empfohlen. Eine Lsung mit dem ActiveX wird immer etwas
langsamer sein, als direkte API Aufrufe.

Es gibt mehrere Mglichkeiten diese Routinen einzubinden:
- Direktes hinzufgen der CAN_API.CPP Datei in Visual C++ Projekte
- Verwenden der CAN_API.DLL, was auch in allen brigen Programmiersprachen funktionieren sollte, 
  getestet wurde diese DLL allerdings nur fr Visual C++ Programme (CATT.EXE)

Der Treiber beinhaltet alle Funktionen fr den BasicCAN und den PeliCAN Modus des SJA1000. 
Der BaiscCAN-Modus ist kompatibel zum PCA82c200, der daher auch mit diesem Treiber funktionieren  
sollte (wurde aber nicht getestet). blicherweise wird so vorgegangen:

-# Eine Callbackfunktion definieren, die durch den Treiber aufgerufen wird, wenn CAN Nachrichten 
  oder andere Ereignisse eintreffen. siehe   \link examples_canEvents.c Beispiel \endlink
-# Beim Programmstart wird canInit() aufgerufen. In canInit()  wird unter anderem die Callbackfunktion (siehe oben) angegeben
-# canOpenDevice() aufrufen. Dadurch wird das CAN Gert erst geffnet
-# CAN Nachrichten senden und empfangen und andere CAN API Funktionen aufrufen (z.B. Register auslesen, beschreiben, ...)
-# canCloseDevice() aufrufen. Das CAN Gert wird dadurch geschlossen
-# Vor Programmende dann canDeInit() aufrufen um die interne Strukturen wieder aufzurumen
*/
/*! \example examples_canEvents.c
  Beispiel einer Callback Routine fr die Treiber Ereignisse.
  Die Callback Routine muss bei canInit() angegeben werden.
*/
#if !CAN_API_FILE_HEADER_INCLUDE
#define CAN_API_FILE_HEADER_INCLUDE

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

struct can_api;



/** \brief Senden aus Hintergrundthreads

  Struktur um einen Hintergundthread zu aktivieren. Der Thread sended dann in regelmigen 
  Abstanden eine CAN Botschaft.
*/
struct can_api_send_thread
{
	HANDLE hThread;         //!< Handle des Thread
	unsigned long dwDelay;	//!< Delay fr Hintergrundthread
	ISRInfo isrCAN;         //!< zu sendende CAN Botschaft
	struct can_api *pMain;  //!< Zeiger auf die Hauptverwaltunsstruktur
	int id;                 //!< intern vergebene Identifikationsnummer
	int stop;               //!< 1=Thread soll gestoppt werden
	struct can_api_send_thread *pNext; //!< Zeiger auf die nchste can_api_send_thread Struktur (verkettete Liste)
};





/** \brief Verwaltungsstruktur, wird in allen Funktionen bentigt

  Verwaltungsstruktur fr alle Routinen. Beim initialisieren mit 
  canInit() wird diese Struktur angelegt.
*/
struct can_api
{
	HANDLE hDevice;
	HWND hWnd;
	HANDLE hThread; ///< Hauptthread (behandelt alle hereinkommenden Nachrichten des Treibers)
	HANDLE hBusLoadThread; ///< Thread zur Berechnung der Buslast (1 mal pro Sekunde)
	volatile int iStopThread;
	HANDLE hSignal;
	int iMode; ///< 0=CAN 2.0A  1=CAN 2.0B
	int iFormat; ///< 0=11 Bit  1=29 Bit
	int dwTimeMultiplier; ///< die diversen Betriebssysteme haben unterschiedliche Auflsungen
	int iShowMessages;
	void *pUserData;
	unsigned long dwAccessCode; /*!< \brief Akzeptanzfilter: Code
								      
									  Da beim Schreiben in die SJA1000 Register zunchst in den
									  Reset-Modus umgeschaltet und deshalb 
									  eine laufende Sendung unterbrochen wird, wird der
									  Wert nur bei einer nderung an den Treiber weitergegeben.
									  Dazu wird der jeweils aktuelle Wert in dieser Struktur
									  gehalten und mit dem neu gewnschten Wert (aus der Applikation oder ActiveX)
									  verglichen. Bei Ungleichheit wird der Wert an den Treiber
									  bergeben.
									*/
	unsigned long dwAccessMask; /*!< \brief Akzeptanzfilter: Mask
								      
									  Da beim Schreiben in die SJA1000 Register zunchst in den
									  Reset-Modus umgeschaltet und deshalb 
									  eine laufende Sendung unterbrochen wird, wird der
									  Wert nur bei einer nderung an den Treiber weitergegeben.
									  Dazu wird der jeweils aktuelle Wert in dieser Struktur
									  gehalten und mit dem neu gewnschten Wert (aus der Applikation oder ActiveX)
									  verglichen. Bei Ungleichheit wird der Wert an den Treiber
									  bergeben.
									*/
	unsigned long dwBaudrate; ///< aktuell eingestellte Baudrate
	unsigned long dwDataLost; ///< Verlorene Messages beim Empfang
	unsigned long dwOldTime; ///< Zur Berechnung der Differenz zwischen zwei Botschaften
	unsigned long dwStartTime; ///< Startzeit (wird in OpenCan() festgestellt)
	volatile double statistikTime; ///< verstrichene Zeit \sa statistikBusload
	volatile unsigned long statistikBits; ///< Bitanzahl whrend statistikTime \sa statistikBusload
	volatile double statistikBusload; /*!< \brief Buslast
									       
										   statistikTime und statistikBits wird in regelmigen
										   Abstnden hergenommen um die Buslast zu berechnen.
										   Da dies innerhalb der Applikation geschieht, ist dieser
										   Wert sehr ungenau.
										*/
	CANPARINFO info;
	int (*events)(struct can_api* c, double absTime, double relTime, ISRInfo *i);
	struct can_api_send_thread *pThread;
};




#ifdef __cplusplus
extern "C" {            /* C Deklarationen fr C++ */
#endif  /* __cplusplus */




/** \brief Initialisieren der CAN Routinen

Bevor die Routinen verwendet werden knnen muss canInit() aufgerufen werden. 
Dieser Funktion wird ein HANDLE auf ein vorhandenes
Fenster bergeben (z.B. das Hauptfenster der Applikation) oder NULL.
Dieses HANDLE wird nur fr MessageBox() Aufrufe bentigt.
Der zweite Parameter ist ein Pointer auf eine Funktion (Callback
Funktion). Diese Funktion wird aus der API Sammlung heraus immer
dann aufgerufen, wenn der CAN Treiber ein Ereignis signalisiert. Diese 
Funktion wird asynchron zu beliebigen Zeiten aufgerufen, dies muss
unter Umst. anden bei der Programmierung ber. ucksichtigt werden: Vorsicht 
bei Zugriffe auf globale oder statische Daten.
Als Ergebnis wird bei Erfolg ein Pointer zurckgegeben, der bei allen
anderen Funktionen angegeben werden muss. Bei Misserfolg wird NULL
zurckgegeben.


  \param hParent   NULL oder Handle auf ein vorhandenes Fenster
  \param pUserData beliebige Anwenderdaten, die API trgt den Wert in
                   die can api-Struktur ein, von wo sie in der Callbackroutine dann
                   z.B. ausgelesen werden kann. Im ActiveX wird hier z.B. die ActiveX Instanz eingetragen.
  \param pEvents   Zeiger auf die Callbackfunktion, 
                   in der Callback Funktion mssen die EV2PRG_... Makros ausgewertet werden

  \return          NULL bei Misserfolg, oder ein Zeiger auf can api Struktur

  \sa EV2PRG_ 
  
  \link examples_canEvents.c
  Beispiel
  \endlink
*/
extern struct can_api * WINAPI canInit(HWND hParent, const void *pUserData,int (*events)(struct can_api* c, double absTime, double relTime, ISRInfo *i));



/** \brief Deinitialisieren der CAN Routinen

Interne Strukturen wieder aufrumen, bevor das Programm beendet wird.

  \param c         Ergebnis eines zuvor erfolgten canInit()
*/
extern void  WINAPI canDeInit(struct can_api *c);




/** \brief CAN Treiber ffnen

Ein Gert ffnen. Damit unter NT ebenfalls das Gert \\\\.\CANPAR
(ohne angehngte Ziffer!) geffnet werden kann, sind Registryeintrge
notwendig. Erst wenn ein Gert geffnet ist, knnen CAN Nachrichten
gesendet und empfangen werden.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param pDevName  Gertename z.B. 
                   \li unter Windows 9x: \\\\.\canpar
                   \li unter Windows NT: \\\\.\canpar oder \\\\.\canpar0 oder \\\\.\canpar1
                     oder \\\\.\canpar2
  \param mode      Modus fr den SJA1000
                   \li 0 = BasicCAN
                   \li 1 = PeliCAN
  \param format    Frame Format
                   \li 0 = Standard Frame Format (11 Bit)
                   \li 1 = Extended Frame Format (29 Bit)
  \param pErr      Zeiger auf ein DWORD. In diesem wird durch GetLastError() 
                   der zuletzt aufgetretene Fehler gespeichert.

  \return          
                   - 1 = Treiber erfolgreich geffnet
                   - 0 = Fehler beim ffnen des Treibers in CreateFile()
                   - -1 = Fehler beim Anlegen eines Events in CreateEvent()
                   - -2 = Fehler beim Anlegen eines Threads in CreateThread()
                   - -3 = Fehler beim berprfen der Treiberversion (Version der CAN API.CPP und Treiber passen nicht)
*/
extern int  WINAPI canOpenDevice(struct can_api *c, const char *pDevName, int mode, int format, DWORD *pErr);




/** \brief CAN Treiber schliessen

  Das zuvor ber canOpenDevice() geffnete Gert wieder schliessen.

  \param c         Ergebnis eines zuvor erfolgten canInit()

  \return          
*/
extern void  WINAPI canCloseDevice(struct can_api *c);




/** \brief Framing Format

Framing Format einstellen. Die Einstellung wird nur im PeliCAN Modus
bernommen.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param format    Frame Format
                   \li 0 = Standard Frames (11 Bit Identifier)
                   \li 1 = Extended Frames (29 Bit Identifier)
*/
extern void  WINAPI canExtendedFrames(struct can_api *c, int format);




/** \brief Register des SJA1000 im Reset Modus auslesen.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param nIndex    Registerindex (0...127)

  \return          Registerwert      
*/
extern unsigned char  WINAPI canGetRegisterRR1(struct can_api *c, unsigned char nIndex);




/** \brief Register des SJA1000 im Reset Modus beschreiben.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param nIndex    Registerindex (0...127)
  \param nNewValue zu schreibender Registerwert
*/
extern void  WINAPI canSetRegisterRR1(struct can_api *c, unsigned char nIndex, unsigned char nNewValue);




/** \brief Register des SJA1000 im Normalmodus auslesen.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param nIndex    Registerindex (0...127)

  \return          Registerwert      
*/
extern unsigned char  WINAPI canGetRegister(struct can_api *c, unsigned char nIndex);




/** \brief Register des SJA1000 im Normalmodus beschreiben.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param nIndex    Registerindex (0...127)
  \param nNewValue zu schreibender Registerwert
*/
extern void  WINAPI canSetRegister(struct can_api *c, unsigned char nIndex, unsigned char nNewValue);




/** \brief CAN Botschaft senden

  CAN Botschaft senden. Wobei die Botschaft in einer ISR Struktur bergeben wird.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param isr       Zeiger auf eine initialisierte Struktur, die die zu sendende CAN Botschaft enthlt

  \return          Im Fehlerfall eine Fehlernummern, die durch GetLastError() ermittelt
                   wurde. Ansonsten 0.
*/
extern int  WINAPI canTransmitDataByISR(struct can_api *c, struct ISRInfo *isr);




/** \brief CAN Botschaft senden

  CAN Botschaft senden. Die Botschaft wird direkt angegeben.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param id        CAN Identifier, es sind Standard Frames und Extended Frames mglich
  \param rtr       Remote Transmission Request
                   \li 0 = Daten senden
                   \li 1 = Remote Transmission Request
  \param dlc       Anzahl Datenbytes in data[] (maximal 8 Bytes sind mglich)
  \param data      Zeiger auf einen Puffer mit den Datenbytes

  \return          Im Fehlerfall eine Fehlernummern, die durch GetLastError() ermittelt
                   wurde. Ansonsten 0.
*/
extern int  WINAPI canTransmitData(struct can_api *c, unsigned long id, int rtr, int dlc, unsigned char* data);




/** \brief Statusregister des SJA1000 auslesen.

  \param c         Ergebnis eines zuvor erfolgten canInit()

  \return          Statusbyte des SJA1000.
*/
extern unsigned char  WINAPI canGetSJA1000Status(struct can_api *c);




/** \brief Kommando an den SJA1000 bergeben

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param command   Kommando (CAN_COMMAND...-Makros) an den Treiber bergeben. Die meisten Kommandos
                   werden dabei direkt an den SJA1000 Baustein weitergeleitet.
				   \sa CAN_COMMAND_

  \return          Im Fehlerfall 0. Ansonsten 1.
*/
extern int  WINAPI canSetSJA1000Command(struct can_api *c, unsigned char command);




/** \brief Hardware Akzeptanzfilter des SJA1000 einstellen

Die Werte werden
direkt an den SJA1000 Baustein eingetragen. Die Werte sind nicht
selbsterklrend, es wird dringend empfohlen, vorher das Datenblatt
zum SJA1000 Baustein zu lesen! Als Alternative kann canAcceptanceFilter() 
verwendet werden.

Die Standardeinstellung ist 0xffffffff fr Code und Mask. Es werden dann keine
Botschaften gefiltert.



  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param code      Akzeptanzkode
  \param mask      Akzeptanzmaske

  \return          Im Fehlerfall 0. Ansonsten 1.
*/
extern int  WINAPI canAcceptanceCodeMask(struct can_api *c, unsigned long code, unsigned long mask);




/** \brief Akzeptanzfilter

  Dieser Akzeptanzfilter ist im Treiber als Softwarelsung
  implementiert. Das heit, der SJA1000 Baustein ist am besten so eingestellt,
  dass alle Botschaften weitergeleitet werden. Die Filterung bernimmt danach
  der Treiber.
  \sa canAcceptanceCodeMask()

  Die Standardeinstellung ist 0 fr Code und Mask. Es werden dann keine
  Botschaften gefiltert.

  Die Filterformel lautet: ((empfangene ID) AND mask) = (code AND mask)
  Bei Gleichheit wird die Botschaft weitergeleitet.

  \version         Ab V2.3
  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param code      Akzeptanzkode
  \param mask      Akzeptanzmaske

  \return          Im Fehlerfall 0. Ansonsten 1.
*/
extern int  WINAPI canAcceptanceFilter(struct can_api *c, unsigned long code, unsigned long mask);




/** \brief Baudrate einstellen

  Die Demoversion erlaubt
  hier nur maximal 150000 Baud.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param baudrate  Baudrate 10000...1000000
  
  \return          Im Fehlerfall 0. Ansonsten 1.
*/
extern int  WINAPI canSetBaudrate(struct can_api *c, unsigned long baudrate);




/** \brief Absolute Zeit rcksetzen.

  \param c         Ergebnis eines zuvor erfolgten canInit()
*/
extern void  WINAPI canResetTime(struct can_api *c);




/** \brief Absolute Zeit ermitteln

Die Zeit ist ein 64 Bit Wert und ist abhngig
vom Betriebsystem. Unter Windows 9x betrgt die Auflsung 800 ns
und unter Windows NT 100 ns. Der 64 Bit Wert wird durch einen
Betriebsystemspezifischen 32 Bit Wert dividiert. Vom Ergebnis wird
der niederwertige 32 Bit Wert als Ergebnis geliefert.

  \param c         Ergebnis eines zuvor erfolgten canInit()

  \return          Zeit in Mikrosekunden.
*/
extern long  WINAPI canGetTime(struct can_api *c);




/** \brief Ermittelt die Anzahl verlorener Nachrichten

  \param c         Ergebnis eines zuvor erfolgten canInit()

  \return          Anzahl der verloren gegangen Nachrichten.
*/
extern long  WINAPI canGetLostDataMsg(struct can_api *c);




/** \brief Liefert als Ergebnis die Buslast. 

Die Buslast wird in CAN API.CPP in
einem parallel laufenden Thread stndig berechnet. Da dies nicht der
Treiber erledigt, ist dieser Wert sehr ungenau!

  \param c         Ergebnis eines zuvor erfolgten canInit()

  \return          Buslast in Prozent
*/
extern double  WINAPI canGetBusLoad(struct can_api *c);




/** \brief Manipulation eines internen Wertes

Hiermit kann die Anzahl der verlorenen Nachrichten manipuliert werden. 
Im Normalfall sollte diese Funktion verwendet werden, um den
Wert auf null rckzusetzen.


  \param c         Ergebnis eines zuvor erfolgten canInit()
*/
extern void  WINAPI canSetLostDataMsg(struct can_api *c, long nNewValue);




/** \brief Ereignisse weitermelden ein/ausschalten

Benachrichtigung beim Eintreffen neuer Nachrichten (empfangene, ge-
sendete CAN Botschaften oder Fehler) ein- oder ausschalten.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param on_off    Ereignisse ein- und ausschalten
                   - 0 = CAN Botschaften nicht anzeigen
                   - 1 = CAN Botschaften anzeigen
*/
extern void  WINAPI canShowMessages(struct can_api *c, int on_off);




/** \brief Thread beenden

Ein zuvor durch canAddTransmitThread() angelegter Thread beenden.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param threadId  Thread ID, die in canAddTransmitThread angegeben wurde

  \return          
                   - 0 = Thread wurde nicht gelscht (existiert nicht)
                   - 1 = Thread wurde gelscht
*/
extern int  WINAPI canDeleteTransmitThread(struct can_api *c, int threadId);




/** \brief Alle Threads beenden

Alle Threads, die durch canAddTransmitThread() angelegt wurden,
wieder beenden.

  \param c         Ergebnis eines zuvor erfolgten canInit()
*/
extern void  WINAPI canDeleteAllTransmitThreads(struct can_api *c);




/** \brief Neuen Thread anlegen

Es wird ein Thread mit der Identifikation threadId angelegt. In einer
festgelegten Periodenzeit (delay) wird dann Endlos lange die angegeben
CAN Nachricht gesendet.

  \param c         Ergebnis eines zuvor erfolgten canInit()
  \param threadId  Thread-Identifier
  \param canId     CAN Identifier der zu sendenden CAN Botschaft
  \param rtr       Remote Transmission Request 
                   \li 0 = Daten senden  
				   \li 1 = Remote Transmission Request
  \param dlc       Anzahl Datenbytes in data (maximal 8 Bytes sind m. oglich)
  \param data      Zeiger auf einen Puffer mit den Datenbytes
  \param delay     Wartezeit in ms zwischen zwei Sendungen

  \return          
                   - 0 = Thread wurde nicht angelegt
                   - 1 = Thread wurde angelegt
*/
extern int  WINAPI canAddTransmitThread(struct can_api *c, int threadId, unsigned long canId, int rtr, int dlc, unsigned char *data, unsigned long delay);

// Neu ab 6. Juli 2002




/** \brief Ermitteln ob CAN Gert geffnet wurde

ber diese Funktion kann ermittelt werden, ob das CAN Gert mit canOpenDevice()
bereits geffnet wurde.

  \version         Ab V2.3
  \param c         Ergebnis eines zuvor erfolgten canInit()

  \return          
                   - 0 = Gert ist noch nicht geffnet
                   - 1 = Gert wurde bereits geffnet
*/
extern int  WINAPI canIsDeviceOpened(struct can_api *c);

#ifdef __cplusplus
}
#endif  /* __cplusplus */


#endif // CAN_API_FILE_HEADER_INCLUDE
